## Exam 2, Problem 4
rm(list=ls(all=TRUE)) # clear the workspace

library(MASS) # Negative Binomial distribution
library(car)  # diagnostic plots
library(AER)  # dispersiontest()

# Load Data:
#   Make sure you have these files in the Working Directory
#   In RStudio file menu: Session > Set Working Directory > Source File Location
testdata=read.csv("ex2-prob4-test.csv")
regdata=read.csv("ex2-prob4-reg.csv")

# MLR model regression
fit1 = lm(contacts ~ . , data=regdata)
summary(fit1)

# Diagnostics
plot(fit1)
# Not good! What are our alternatives? 

# Note these are COUNT DATA! Try poisson GLM, negative binomial GLM

## Poisson GLM with log link function:
fit2 = glm(contacts ~ . , data=regdata, family = poisson)

# Quick Poisson check: Does mean = variance? 
c(mean=mean(regdata$contacts), variance=var(regdata$contacts), sd=sd(regdata$contacts))
# This may suggest the data are overdispersed,
# BUT more carefully check for overdispersion in a regression context.
# Use AER::dispersiontest() on the GLM Poisson object:
dispersiontest(fit2)
# dispersion >0 means overdispersed (Variance > Mean)
# dispersion <0 means underdispersed (Variance < Mean)

## OVERDISPERSED DATA: What can we do about overdispersed data? 
## First option: QuasiPoisson GLM, but this has limitations!!!
fit3 = glm(contacts ~ . , data=regdata, family = quasipoisson)

## Second alternative: Negative Binomial GLM (Preferred over quasipoisson)
#
#   NOTE: glm(...,family=negative.binomial) requires a known parameter theta!  
#   Why? In short, glm() works for "exponential family" distributions.
#   Some distributions are only in the exponential family when 
#   we assume certain parameters are known...
#   Ex:  Binomial, where n is assumed to be known.
#

# SOLUTION: Estimate theta using a crude iterative method: 
#       First:  guess theta, 
#       Second: estimate regression coefficients, 
#       Third:  re-estimate theta...

# First, guess some theta values...
fit4a = glm(contacts ~ . , data=regdata, family = negative.binomial(theta=0.5))
fit4b = glm(contacts ~ . , data=regdata, family = negative.binomial(theta=5))

# Next, use that fit to estimate theta. Note we have options to choose from...
theta.est.a=c(ML=theta.ml(fit4a)[1], Md=theta.md(fit4a), MM=theta.mm(fit4a))
theta.est.b=c(ML=theta.ml(fit4b)[1], Md=theta.md(fit4b), MM=theta.mm(fit4b))
theta.est.a # see ?theta.ml for details on the different estimates
theta.est.b # Similar estimates between case a and case b! :-)

# Use the maximum likelihood (ML) estimate in our GLM. 
fit4c = glm(contacts ~ . , data=regdata, family = negative.binomial(theta=theta.ml(fit4a)))

# Whew! BUT, wouldn't it be nice if glm() just estimated theta for us? Well...

# ALTERNATIVE way to use a Neg. Binomial GLM: 
#    Use the MASS::glm.nb() function to estimate theta internally!
fit4 = glm.nb(contacts ~ . , data=regdata) 
# If needed, use glm.convert() to make fit4 the same 
# type of object as returned by glm(). 
# fit4.glm = glm.convert(fit4)
# We can just use fit4c above instead.

###############################################################################
# Now fit2, fit3, and fit4 contain our three GLM models (all with log links).

# Note each model yields the same coefficient estimates
coefficients(fit2) # Poisson
coefficients(fit3) # QuasiPoisson
coefficients(fit4) # Negative Binomial 

# Error/noise structures, however, differ
summary(fit2)           
summary(fit3)           
summary(fit4)

#######################  DIAGNOSTICS  #######################
# Plot diagnostics for all three log-link GLM models
x11() # maximize this window to fill your screen!!
par(mfrow=c(3,5))
# Prediction interval: 95%
alpha=1-0.95

##### Usual diagnostics plus prediction intervals for each model

## Row 1: POISSON
plot(fit2, which=c(1,3,5,2), main="Poisson")
plot(testdata$contacts, 
     xlab="ID",ylab="Contacts (95% PI)", ylim=c(0,65), 
     main="Predictions | Poisson")
pmeans = predict(fit2, testdata[-1], type="response") # means for Poisson
# loop through each point, plot prediction interval as a vertical line
lower=c()
upper=c()
for(i in 1:nrow(testdata)) {   
  lower[i]=qpois(alpha/2, pmeans[i])
  upper[i]=qpois(1-alpha/2, pmeans[i])
  points(c(i,i), c(lower[i], upper[i]), col="red", type="l")
} 
points(testdata$contacts, pch=19) # redraw points
# What percent of testdata in the 95% prediction interval?
sum(lower <= testdata$contacts & testdata$contacts <= upper)/nrow(testdata)*100

## Row 2: QUASIPOISSON -- no true prediction intervals!!!
plot(fit3, which=c(1,3,5,2), main="QuasiPoisson")
plot(testdata$contacts, 
     xlab="ID",ylab="Contacts (95% PI)", ylim=c(0,65), 
     main="Predictions | QuasiPoisson")
text(25,50,"No real distribution, no Prediction Intervals! :-(")
  

## Row 3: NEGATIVE BINOMIAL
plot(fit4, which=c(1,3,5,2), main="Negative Binomial")
plot(testdata$contacts, 
     xlab="ID",ylab="Contacts (95% PI)", ylim=c(0,65), 
     main="Predictions | Negative Binomial")
pmeans = predict(fit4, testdata[-1], type="response") # means for Poisson
# loop through each point, plot prediction interval as a vertical line
lower=c()
upper=c()
for(i in 1:nrow(testdata)) {   
  lower[i]=qnbinom(alpha/2,  mu=pmeans[i], size=fit4$theta)
  upper[i]=qnbinom(1-alpha/2,  mu=pmeans[i], size=fit4$theta)
  points(c(i,i), c(lower[i], upper[i]), col="red", type="l")
} 
points(testdata$contacts, pch=19) # redraw points
# What percent of testdata in the 95% prediction interval?
sum(lower <= testdata$contacts & testdata$contacts <= upper)/nrow(testdata)*100

